#!/usr/bin/env python3

from clamps import Joystick

def main():
    import time

    js = Joystick(0)
    n = 0

    while js.valid:
        try:
            js.dump_axes()
            js.dump_buttons()

            print(f"{n} ----------------------\n")
            time.sleep(0.1)
            # if ps4.buttons.pad is True:
            #     break

            # js.dump_buttons()
            # js.dump_axes()
            n += 1

        except KeyboardInterrupt:
            print('js exiting ...')
            break


if __name__ == "__main__":
    main()


# -*- coding: utf-8 -*-
##############################################
# The MIT License (MIT)
# Copyright (c) 2014 Kevin Walchko
# see LICENSE for full details
##############################################
from .js import PS4Joystick
from .js import PS5Joystick
from .js import Joystick
from .js import Axis, PSButtons, JS, JSInfo

from importlib.metadata import version # type: ignore

__author__ = "Kevin Walchko"
__license__ = "MIT"
__version__ = version("clamps")


#!/usr/bin/env python3
#
# by Kevin J. Walchko 26 Aug 2014
#
# PS4 has 6 axes, 16 buttons, 0 hats (not recognized)

try:
    # brew install sdl2
    # pip install PySDL2
    import sdl2
except ImportError:
    print('You must install SLD2 library for this to work')
    print("  macos: brew install sdl2")
    print("  linux: apt install libsdl-dev")
    print("  pip install PySDL2")

    import sys
    sys.exit(1)

from collections import namedtuple

Axis = namedtuple("Axis", "x y")
PSButtons = namedtuple("PSButtons",
    "x circle square triangle share ps options L3 R3 L1 R1 dp_up dp_down dp_left dp_right pad")
JS = namedtuple("JS", "info leftstick rightstick triggers buttons")
JSInfo = namedtuple("JSInfo", "num_buttons num_axes num_hats")

class Joystick:
    def __init__(self, num):
        # init SDL2 and grab joystick
        sdl2.SDL_Init(sdl2.SDL_INIT_JOYSTICK)
        self.js = sdl2.SDL_JoystickOpen(num)

        axes = sdl2.SDL_JoystickNumAxes(self.js)
        btns = sdl2.SDL_JoystickNumButtons(self.js)
        hats = sdl2.SDL_JoystickNumHats(self.js)

        self.info = JSInfo(btns, axes, hats)

        if axes < 1:
            print('*** No Joystick found ***')
            self.valid = False
        else:
            print('==========================================')
            print(' PS4 Joystick ')
            print(f'   axes: {self.info.num_axes} buttons: {self.info.num_buttons} hats: {self.info.num_hats}')
            print('==========================================')
            self.valid = True

    def __del__(self):
        # clean-up
        if self.valid:
            self.close()
        print('Bye ...')

    def close(self):
        sdl2.SDL_JoystickClose(self.js)
        self.valid = False

    def dump_buttons(self):
        # debug buttons
        print("Buttons")
        sdl2.SDL_JoystickUpdate()
        for i in range(self.info.num_buttons):
            print(f"  {i}: {sdl2.SDL_JoystickGetButton(self.js, i)}")

    def dump_axes(self):
        # debug axes
        print("Axes")
        sdl2.SDL_JoystickUpdate()
        for i in range(self.info.num_axes):
            print(f"  {i}: {sdl2.SDL_JoystickGetAxis(self.js, i) / 32768}")


class PS4Joystick(Joystick):
    """
    Joystick class setup to handle a Playstation PS4 Controller. If no joystick
    is found, the self.valid is False. If it is not valid, then any returned
    dictionaries will contain all 0 values.

    Buttons     Index number
    --------------------------
        X       = 0
        Circle  = 1
        Square  = 2
        Triangle= 3
        Share   = 4
        PS      = 5
        Options = 6
        L3      = 7
        R3      = 8
        L1      = 9
        R1      = 10
        DPup    = 11
        DPdown  = 12
        DPleft  = 13
        DPright = 14
        PadPress= 15

    Axes
    ---------------------------
        LeftStickX      = X-Axis
        LeftStickY      = Y-Axis (Inverted?)
        RightStickX     = 3rd Axis
        RightStickY     = 4th Axis (Inverted?)
        L2              = 5th Axis (-1.0f to 1.0f range, unpressed is -1.0f, shifted to 0-2)
        R2              = 6th Axis (-1.0f to 1.0f range, unpressed is -1.0f, shifted to 0-2)

    WARNING: doesn't work as a process
    """
    def __init__(self, num=0):
        super().__init__(num)

    def get(self, raw=False):
        js = self.js

        sdl2.SDL_JoystickUpdate()

        # these don't work :(
        # accels
        # x = sdl2.SDL_JoystickGetAxis(js, 6) / 32768
        # y = sdl2.SDL_JoystickGetAxis(js, 7) / 32768
        # z = sdl2.SDL_JoystickGetAxis(js, 8) / 32768
        # ps4["axes"] = [x, y, z]
        #
        # # gyros
        # x = sdl2.SDL_JoystickGetAxis(js, 9) / 32768
        # y = sdl2.SDL_JoystickGetAxis(js, 10) / 32768
        # z = sdl2.SDL_JoystickGetAxis(js, 11) / 32768
        # ps4.axes.gyros = [x, y, z]
        #
        # # get hat - doesn't work
        # # [up right down left] = [1 2 4 8]
        # print("hat", sdl2.SDL_JoystickGetHat(js, 0), "\n")
        # print('b 13', sdl2.SDL_JoystickGetButton(js, 13))

        vals = [True if sdl2.SDL_JoystickGetButton(js, i) else False for i in range(self.info.num_buttons)]
        buttons = PSButtons(*vals[:16])

        if raw:
            ps4 = JS(
                self.info, # joystick info
                Axis( # left stick
                    sdl2.SDL_JoystickGetAxis(js, 0),
                    sdl2.SDL_JoystickGetAxis(js, 1)
                ),
                Axis( # right stick
                    sdl2.SDL_JoystickGetAxis(js, 2),
                    sdl2.SDL_JoystickGetAxis(js, 3)
                ),
                Axis( # triggers L2 R2
                    sdl2.SDL_JoystickGetAxis(js, 4) + 32768,
                    sdl2.SDL_JoystickGetAxis(js, 5) + 32768
                ),
                buttons
            )
        else:
            ps4 = JS(
                self.info, # joystick info
                Axis( # left stick
                    sdl2.SDL_JoystickGetAxis(js, 0) / 32768,
                    sdl2.SDL_JoystickGetAxis(js, 1) / 32768
                ),
                Axis( # right stick
                    sdl2.SDL_JoystickGetAxis(js, 2) / 32768,
                    sdl2.SDL_JoystickGetAxis(js, 3) / 32768
                ),
                Axis( # triggers L2 R2
                    sdl2.SDL_JoystickGetAxis(js, 4) / 32768 + 1,
                    sdl2.SDL_JoystickGetAxis(js, 5) / 32768 + 1
                ),
                buttons
            )

        return ps4

class PS5Joystick(PS4Joystick):
    """
    So there is no difference between PS4 and PS5 joysticks. The 5 has 1 extra buttone
    but I don't enable it ... it is the audio mute LED button at the very back. It didn't
    seem worth the effort to duplicate code just for that.
    """
    pass

def main():
    import time

    # js = PS4Joystick()
    js = PS5Joystick()

    while js.valid:
        try:
            ps4 = js.get()
            print(ps4,"\n----------------------\n")
            time.sleep(0.1)
            if ps4.buttons.pad is True:
                break

            # js.dump_buttons()
            # js.dump_axes()

        except KeyboardInterrupt:
            print('js exiting ...')
            break


if __name__ == "__main__":
    main()


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['clamps', 'clamps.bin']

package_data = \
{'': ['*']}

install_requires = \
['PySDL2']

setup_kwargs = {
    'name': 'clamps',
    'version': '0.1.3',
    'description': 'SDL2 playstation joystick driver for python',
    'long_description': '\n![](https://github.com/MomsFriendlyRobotCompany/clamps/blob/main/clamps.png?raw=true)\n\n# clamps\n\n![GitHub](https://img.shields.io/github/license/MomsFriendlyRobotCompany/clamps)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/clamps)\n![PyPI](https://img.shields.io/pypi/v/clamps)\n![PyPI - Downloads](https://img.shields.io/pypi/dm/clamps?color=aqua)\n\n**Note:** I only have playstations, so I don\'t know if this will work with\nother joysticks.\n\nThis is actually something I did long ago, but finally decided to make\nit a module available on PyPi.\n\n## Setup\n\nThis requires `sdl2` inorder to work:\n\n```bash\nbrew install sdl2       # macos\napt install libsdl2-dev # linux\n```\n\nThis package will install the python bindings `PySDL2` from pypi.\n\nTested:\n\n- Works with joystick connected via USB cable\n- Works with joystick connected via bluetooth\n- Works with PS4 controller\n- Works with PS5 controller\n\n## Example\n\nSo really, there is no difference between `PS4Joystick` amd `PS5Joystick`,\nbut both classes exist. Originally I hoped there would be a difference,\nlike access to the gyros and accelerometers, but there isn\'t.\n\n```python\n#!/usr/bin/env python3\nimport time\nfrom clamps import PS4Joystick\n\ndef main():\n    import time\n\n    js = PS4Joystick() # or PS5Joystick()\n\n    while js.valid:\n        try:\n            ps4 = js.get()\n            print(ps4,"\\n----------------------\\n")\n            time.sleep(0.1)\n            if ps4.buttons.pad is True:\n                break\n        except KeyboardInterrupt:\n            print(\'js exiting ...\')\n            break\n\n\nif __name__ == "__main__":\n    main()\n```\n\nOutput of `get()` is a `namedtuple` with the following fields:\n\n```python\nJS(info=JSInfo(num_buttons=16, num_axes=6, num_hats=0), leftstick=Axis(x=-0.003936767578125, y=0.011749267578125), rightstick=Axis(x=-0.01177978515625, y=-0.050994873046875), triggers=Axis(x=0.0, y=0.0), buttons=PS4Buttons(x=False, circle=False, square=False, triangle=False, share=False, ps=False, options=False, L3=False, R3=False, L1=False, R1=False, dp_up=False, dp_down=False, dp_left=False, dp_right=False, pad=True))\n```\n\n## [How to pair a DUALSHOCK 4 wireless controller with a supported device][ref]\n\nTo pair your wireless controller with a supported device using Bluetooth for the first time, turn on pairing mode.\n\n1. Make sure the player indicator on the controller is off.\nIf the player indicator is on, press and hold the PS button until it turns off. If a USB is connected to the controller, disconnect it.\n1. While pressing and holding the SHARE button, press and hold the PS Button until the light bar flashes.\n1. Enable Bluetooth on your device, and then select the controller from the list of Bluetooth devices. When pairing is complete, the light bar blinks, and then the player indicator lights up.\n\n\n![](https://github.com/MomsFriendlyRobotCompany/clamps/blob/main/js.webp?raw=true)\n\n[ref]: https://www.playstation.com/en-us/support/hardware/ps4-pair-dualshock-4-wireless-with-pc-or-mac/\n\n\n\n# MIT License\n\n**Copyright (c) 2014 Kevin Walchko**\n\nPermission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.\n',
    'author': 'walchko',
    'author_email': 'walchko@users.noreply.github.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://pypi.org/project/clamps/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8',
}


setup(**setup_kwargs)
